@@ -27,14 +27,20 @@
 
 #if PLAINGTK == 0
 # if HILDON == 1
+#  if MAEMO_VERSION_MAJOR == 5
+#     include <hildon/hildon.h>
+#  endif
 #  include <hildon/hildon-program.h>
 #  include <hildon/hildon-banner.h>
+#  include <hildon/hildon-wizard-dialog.h>
 #  include <hildon/hildon-note.h>
 #  include <hildon/hildon-number-editor.h>
 #  include <hildon/hildon-date-editor.h>
 #  include <hildon-fm-2/hildon/hildon-file-chooser-dialog.h>
 #  include <hildon-fm-2/hildon/hildon-file-system-model.h>
-#  include <hildon/hildon-help.h>
+#  if LIBHELP == 1
+#    include <hildon/hildon-help.h>
+#  endif
 # else
 #  include <hildon-widgets/hildon-program.h>
 #  include <hildon-widgets/hildon-banner.h>
@@ -121,8 +127,10 @@
 
 #define YSCALE(y) ((y)*(400.0/700.0*GTK_PLOT_A4_W/GTK_PLOT_A4_H))
 
+#define DIALOG_MIN_HEIGHT0 300
 #define DIALOG_MIN_HEIGHT1 200
 #define DIALOG_MIN_HEIGHT2 150
+#define DIALOG_MIN_HEIGHTMAX 400
 #define DIALOG_MIN_WIDTH1 720
 
 enum
@@ -183,7 +191,11 @@ struct _AppData
 {
 #if PLAINGTK == 0
   HildonProgram *program;     /* handle to application */
+#if MAEMO_VERSION_MAJOR == 5
+  GtkWidget *mainwindow;   /* handle to main window */
+#else
   HildonWindow *mainwindow;   /* handle to main window */
+#endif
   HildonWindow *statwindow;   /* handle to statistics window */
   HildonWindow *reportwindow; /* handle to report window */
   HildonWindow *alarmwindow;  /* handle to alarm window */
@@ -208,6 +220,9 @@ struct _AppUIData
   gboolean reportfullscreen;
   gboolean alarmfullscreen;
 
+  /* Warnings active */
+  gboolean warn;
+
   /* Main window */
   GtkWidget *mainvbox;
   GtkWidget *view;
@@ -227,7 +242,11 @@ struct _AppUIData
   GtkWidget *alarm_menu_item_fullscreen;
 
   /* Main menu */
+#if MAEMO_VERSION_MAJOR == 5
+  HildonAppMenu *main_menu;
+#else
   GtkWidget *main_menu;
+#endif
   GtkWidget *mm_menu_db;
   GtkWidget *mm_menu_record;
   GtkWidget *mm_menu_stat;
@@ -256,6 +275,7 @@ struct _AppUIData
   GtkWidget *mm_item_delete;
 
   /* Statistics menu (in main menu) */
+  GtkWidget *mm_item_quick;
   GtkWidget *mm_item_monthly;
   GtkWidget *mm_item_report;
 
@@ -281,6 +301,20 @@ struct _AppUIData
 #endif
 
   /* Settings dialog */
+#if MAEMO_VERSION_MAJOR == 5
+  GtkWidget *databasebutton;
+  GtkWidget *carbutton;
+  GtkWidget *driverbutton;
+  GtkWidget *datebutton;
+  GtkWidget *remindbutton;
+  GtkWidget *paperbutton;
+  GtkWidget *unitsystembutton;
+  GtkWidget *lengthunitbutton;
+  GtkWidget *volumeunitbutton;
+  GtkWidget *consumeunitbutton;
+  GtkWidget *massunitbutton;
+  GtkWidget *fueltypebutton;
+#endif
   GtkWidget *drivercombo;
   GtkWidget *carcombo;
   GtkWidget *datecombo;
@@ -315,7 +349,9 @@ struct _AppUIData
   GtkWidget *stb_totalkmlabel;
   GtkWidget *stb_avgconsumlabel;
   GtkWidget *stb_totcostlabel;
-
+#if MAEMO_VERSION_MAJOR == 5
+  GtkWidget *stb_carbutton;
+#endif
   /* Add driver dialog */
   GtkWidget *entryname;
   GtkWidget *entrynick;
@@ -333,6 +369,7 @@ struct _AppUIData
   GtkWidget *entrykm;
   GtkWidget *entrytrip;
   GtkWidget *entryfill;
+  GtkWidget *buttonnotfull;
   GtkWidget *entryprice;
   GtkWidget *entrynotes;
   GtkWidget *entryservice;
@@ -343,17 +380,29 @@ struct _AppUIData
   GtkWidget *columncheck[NUM_COLS];
 
   /* Filter records dialog */
-  GtkWidget *addbutton[NUM_COLS];
-  GtkWidget *andorcombo[NUM_COLS];
-  GtkWidget *filterminentry[NUM_COLS];
-  GtkWidget *filtermaxentry[NUM_COLS];
-  GtkWidget *filterentry;
+  GtkWidget *filterwindow; 
+#if MAEMO_VERSION_MAJOR == 5
+  GtkWidget *filteraddandorbutton;
+#endif
+
+  /* Add filter dialog */
+  GtkWidget *addfilterdialog;
+  GtkWidget *filterbutton;
+  GtkWidget *filtercombo;
+  GtkWidget *filtervbox;
+  GtkWidget *filtertable;
+  GtkWidget *filteraddandorcombo;
+  GtkWidget *filteraddminentry;
+  GtkWidget *filteraddmaxentry;
 
   /* Report window */
   GtkWidget *reportcanvas;
   GtkWidget *reportscrollwin;
   GtkWidget *yearcombo;
   GtkWidget *papercombo;
+#if MAEMO_VERSION_MAJOR == 5
+  GtkWidget *lastyearbutton;
+#endif
 
   /* Import window */
   GtkWidget *colspin[IMPORTCOLS];
@@ -368,12 +417,16 @@ struct _AppUIData
   GtkWidget *active_plot;
   GtkPlotArray *months;
   GtkPlotCanvasChild *maintitle;
+  GtkPlotCanvasChild *subtitle;
 
   /* Statistics window menu */
   GtkWidget *sm_item_monthly;
   GtkWidget *sm_item_monthlytrip;
   GtkWidget *sm_item_monthlypriceperlitre;
   GtkWidget *sm_item_consumdistr;
+#if MAEMO_VERSION_MAJOR == 5
+  GtkWidget *sm_stat_combo;
+#endif
 
   /* Alarm window */
   GtkWidget *alarmview;
@@ -390,6 +443,9 @@ struct _AppUIData
   GtkWidget *entryeventtires;
   GtkWidget *entryeventnotes;
   GtkWidget *eventview;
+#if MAEMO_VERSION_MAJOR == 5
+  GtkWidget *alarmcarbutton;
+#endif
 
   /* Alarm secondary toolbar */
   GtkToolItem *stb_alarmdesc;
@@ -426,9 +482,21 @@ gchar * interface_file_chooser (gchar *header,
 				GtkFileChooserAction action, 
 				const gchar *ext,
 				gchar *help_id);
-GtkWidget *create_driver_combo(void);
-GtkWidget *create_car_combo(void);
+#if MAEMO_VERSION_MAJOR == 5
+GtkWidget *create_driver_combo(AppUIData *pui, GtkWidget *button);
+#else
+GtkWidget *create_driver_combo(AppUIData *pui);
+#endif
+#if MAEMO_VERSION_MAJOR == 5
+GtkWidget *create_car_combo(AppUIData *pui, GtkWidget *button);
+#else
+GtkWidget *create_car_combo(AppUIData *pui);
+#endif
+#if MAEMO_VERSION_MAJOR == 5
+GtkWidget *create_unit_combo(gint whichunit, AppUIData *pui, GtkWidget *button);
+#else
 GtkWidget *create_unit_combo(gint whichunit, AppUIData *pui);
+#endif
 double SIlength2user(double length);
 double user2SIlength(double length);
 double SIvolume2user(double length);
@@ -438,12 +506,14 @@ double user2SIconsumption(double consum);
 double SIppl2user(double ppl);
 double SImass2user(double mass);
 double SIemission2user(double emission);
+double grams2kg(double grams);
 void update_record_model(AppUIData *pui);
 void update_totalkm(AppUIData *pui);
 void update_reminder_toolbutton(AppUIData *pui, gboolean warn);
 void update_car_changed(AppUIData *pui);
 void help_activated(GtkWidget *win, gchar *help_id);
 void get_store_and_iter(GtkTreeModel *model, GtkWidget *view, GtkTreeIter *iter, GtkListStore **store, GtkTreeIter *storeiter);
+GString *get_entry(GtkWidget *entry);
 
 #if PLAINGTK == 0
 void help_dialog_help_enable(GtkDialog *dialog, const gchar *topic, osso_context_t *osso);
