@@ -40,6 +40,8 @@
 #define COSTPERKMTAG "COSTPERKMTAG"
 #define COSTTAG "COSTTAG"
 #define PRICEPERUNITTAG "PRICEPERUNITTAG"
+#define TOTALCOSTTAG "TOTALCOSTTAG"
+#define TOTALCOSTPERKMTAG "TOTALCOSTPERKMTAG"
 
 extern unit_t curunit;
 extern gboolean carchanged;
@@ -155,7 +157,46 @@ void create_reportwin_menu(HildonWindow *window, AppUIData *pui)
   GtkWidget *menubar;
   GtkWidget *item_file;
 #endif
- 
+
+#if MAEMO_VERSION_MAJOR == 5
+  main_menu = HILDON_APP_MENU (hildon_app_menu_new ());
+
+  item_save = hildon_button_new_with_text(HILDON_SIZE_AUTO,
+					  HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+					  _("Save report"),
+					  NULL);
+  hildon_app_menu_append (main_menu, GTK_BUTTON (item_save));
+
+  item_exit = hildon_button_new_with_text(HILDON_SIZE_AUTO,
+					  HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+					  _("Close"),
+					  NULL);
+  hildon_app_menu_append (main_menu, GTK_BUTTON (item_exit));
+
+  item_zoom_in = hildon_button_new_with_text(HILDON_SIZE_AUTO,
+					     HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+					     _("Zoom in"),
+					     NULL);
+  hildon_app_menu_append (main_menu, GTK_BUTTON (item_zoom_in));
+
+  item_zoom_out = hildon_button_new_with_text(HILDON_SIZE_AUTO,
+					      HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+					      _("Zoom out"),
+					      NULL);
+  hildon_app_menu_append (main_menu, GTK_BUTTON (item_zoom_out));
+
+  g_signal_connect(G_OBJECT(item_save), "clicked",
+		   G_CALLBACK(callback_save_report), pui->reportcanvas);
+  g_signal_connect_swapped(G_OBJECT(item_exit), "clicked",
+			   G_CALLBACK(delete_reportwin), G_OBJECT(window));
+  g_signal_connect(G_OBJECT(item_zoom_in), "clicked",
+		   G_CALLBACK(callback_report_zoom_in), pui->reportcanvas);
+  g_signal_connect(G_OBJECT(item_zoom_out), "clicked",
+		   G_CALLBACK(callback_report_zoom_out), pui->reportcanvas);
+
+  hildon_window_set_app_menu(HILDON_WINDOW(window), main_menu);
+
+#else 
   main_menu = gtk_menu_new();
   menu_view = gtk_menu_new();
   menu_zoom = gtk_menu_new();
@@ -219,7 +260,7 @@ void create_reportwin_menu(HildonWindow *window, AppUIData *pui)
 		   G_CALLBACK(callback_report_zoom_in), pui->reportcanvas);
   g_signal_connect(G_OBJECT(item_zoom_out), "activate",
 		   G_CALLBACK(callback_report_zoom_out), pui->reportcanvas);
-#if PLAINGTK == 0
+#if LIBHELP == 1
   g_signal_connect(G_OBJECT(item_help), "activate",
 		   G_CALLBACK(help_activated), HELP_ID_REPORT);
 #endif
@@ -228,7 +269,7 @@ void create_reportwin_menu(HildonWindow *window, AppUIData *pui)
 
   g_signal_connect(G_OBJECT(pui->report_menu_item_fullscreen), "toggled",
 		   G_CALLBACK(cb_report_fullscreen), pui);
-
+#endif
   /* Make all menu widgets visible */
   gtk_widget_show_all(GTK_WIDGET(main_menu));
 }
@@ -398,29 +439,47 @@ void callback_report_zoom_out( GtkAction * action, gpointer data )
  */
 void fill_year_combo(GtkWidget *combo)
 {
-  char listitemtext[ENTRYDATEMAX]="";
+  GString *listitemtext;
   gint i;
 
   i=0;
+  listitemtext=g_string_new(NULL);
 
   if (SQLITE_OK == sqlite3_bind_int(ppStmtGetYears, 1, currentcar)) {
     while(SQLITE_ROW == sqlite3_step(ppStmtGetYears)) {
-      g_snprintf(listitemtext, ENTRYDATEMAX, "%s", 
-		 sqlite3_column_text(ppStmtGetYears,0));
-      gtk_combo_box_append_text(GTK_COMBO_BOX (combo),listitemtext);
+      g_string_printf(listitemtext, "%s", 
+		      sqlite3_column_text(ppStmtGetYears,0));
+#if MAEMO_VERSION_MAJOR == 5
+      hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (combo),listitemtext->str);
+#else
+      gtk_combo_box_append_text(GTK_COMBO_BOX (combo),listitemtext->str);
+#endif
       i++;
     }
     sqlite3_reset(ppStmtGetYears);
+#if MAEMO_VERSION_MAJOR == 5
+    hildon_touch_selector_set_active(HILDON_TOUCH_SELECTOR (combo), 0, i-1);
+#else
     gtk_combo_box_set_active(GTK_COMBO_BOX (combo),i-1);
+#endif
   }
+  g_string_free(listitemtext, TRUE);
 }
 
-GtkWidget *create_year_combo(void)
+GtkWidget *create_year_combo(AppUIData *pui)
 {
   GtkWidget *combo;
 
+#if MAEMO_VERSION_MAJOR == 5
+  combo = hildon_touch_selector_new_text ();
+#else
   combo = gtk_combo_box_new_text();
+#endif
   fill_year_combo(combo);
+#if MAEMO_VERSION_MAJOR == 5
+  hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (pui->lastyearbutton),
+                                     HILDON_TOUCH_SELECTOR (combo));
+#endif
   return combo;
 }
 
@@ -436,13 +495,27 @@ GtkWidget *create_year_combo(void)
  */
 void update_yearcombo_cb(GtkWidget *widget, gpointer combo)
 {
+#if MAEMO_VERSION_MAJOR == 5
+  HildonTouchSelectorColumn *column;
+#endif
   GtkTreeModel *model;
   gint i, count;
 
+#if MAEMO_VERSION_MAJOR == 5
+  model = hildon_touch_selector_get_model(HILDON_TOUCH_SELECTOR(combo), 0);
+  gtk_list_store_clear(GTK_LIST_STORE(model));
+/*   hildon_touch_selector_remove_column (HILDON_TOUCH_SELECTOR(combo), 0);   */
+/* /\*   combo = hildon_touch_selector_new_text (); *\/ */
+/*   column = hildon_touch_selector_append_text_column (HILDON_TOUCH_SELECTOR(combo), */
+/* 						     model, */
+/* 						     TRUE); */
+/*   hildon_touch_selector_column_set_text_column(column, 0); */
+#else
   model = gtk_combo_box_get_model(GTK_COMBO_BOX(combo));
   count = gtk_tree_model_iter_n_children(model, NULL);
   for (i=0; i < count; i++)
     gtk_combo_box_remove_text(GTK_COMBO_BOX(combo), 0);
+#endif
   fill_year_combo(GTK_WIDGET(combo));
 }
 
@@ -525,9 +598,8 @@ gchar *report_conv_cost_label(const char *s)
   }
   if (strncmp(CO2TAG,s,strlen(CO2TAG))==0) {
     if (curunit.massunit==SI) {
-      if (curunit.lengthunit==SI)
-	return g_strconcat(_("Avg. CO2"), " ", (curunit.lengthunit==SI)? 
-			   _("g/km") : _("g/mile"), NULL);
+      return g_strconcat(_("Avg. CO2"), " ", (curunit.lengthunit==SI)? 
+			 _("g/km") : _("g/mile"), NULL);
     }
     else
       return g_strconcat(_("Avg. CO2"), " ", (curunit.lengthunit==SI)? 
@@ -535,8 +607,7 @@ gchar *report_conv_cost_label(const char *s)
   }
   if (strncmp(TOTALCO2TAG,s,strlen(TOTALCO2TAG))==0) {
     if (curunit.massunit==SI) {
-      if (curunit.lengthunit==SI)
-	return g_strconcat(_("Tot. CO2"), " ",  _("kg"), NULL);
+      return g_strconcat(_("Tot. CO2"), " ",  _("kg"), NULL);
     }
     else
       return g_strconcat(_("Tot. CO2"), " ",_("lb"), NULL);
@@ -547,6 +618,15 @@ gchar *report_conv_cost_label(const char *s)
     return g_strconcat(_("Cost"), " ", curunit.currency->str, "/",
 		       (curunit.lengthunit==SI)? _("km") : _("mile"), 
 		       NULL);
+  if (strncmp(TOTALCOSTTAG,s,strlen(TOTALCOSTTAG))==0)
+      return g_strconcat(_("Total cost"), " ",  curunit.currency->str, NULL);
+  if (strncmp(TOTALCOSTPERKMTAG,s,strlen(TOTALCOSTPERKMTAG))==0) {
+    if (curunit.lengthunit==SI) {
+      return g_strconcat(_("Total cost"), " ", curunit.currency->str, "/",  _("km"), NULL);
+    }
+    else
+      return g_strconcat(_("Total cost"), " ", curunit.currency->str, "/",  _("mile"), NULL);
+  }
   else
     return g_strdup(s);
 }
@@ -562,13 +642,14 @@ void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
   gchar *cartexts[] = {gettext_noop("Car"),
 			  gettext_noop("Car"),
 			  gettext_noop("Model year"),
-			  gettext_noop("Register"});
+			  gettext_noop("Register")};
   gchar *costtexts[] = {gettext_noop("Overall expenses"),
 			  gettext_noop("Fuel"), 
 			  gettext_noop("Oil"),
 			  gettext_noop("Service"), 
 			  gettext_noop("Tires"),
-			  gettext_noop("Total cost")};
+			TOTALCOSTTAG,
+			TOTALCOSTPERKMTAG};
   gchar *kmtexts[] = {gettext_noop("Distance"), 
 			gettext_noop("Beginning of year"), 
 			gettext_noop("End of year"), 
@@ -594,59 +675,84 @@ void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 
   gint lastyear;
   gchar *lastyearbuf;
-  gchar car[30];
-  gchar modelyear[30];
-  gchar reg[30];
-  gchar year[ENTRYDATEMAX];
-  gchar startkm[ENTRYKMMAX];
-  gchar endkm[ENTRYKMMAX];
-  gchar yearkm[ENTRYKMMAX];
-  gchar sumfill[ENTRYKMMAX]; 
-  gchar sumprice[ENTRYKMMAX];
-  gchar avgconsum[ENTRYKMMAX];
-  gchar avgco2[ENTRYKMMAX];
-  gchar avgprice[ENTRYKMMAX];
-  gchar totalco2[ENTRYKMMAX];
-  gchar pricekm[ENTRYKMMAX];
-  gchar priceoil[ENTRYKMMAX];
-  gchar priceoilkm[ENTRYKMMAX];
-  gchar priceservice[ENTRYKMMAX];
-  gchar priceservicekm[ENTRYKMMAX];
-  gchar pricetires[ENTRYKMMAX];
-  gchar pricetireskm[ENTRYKMMAX];
-  gchar totalprice[ENTRYDATEMAX];
+  GString *car;
+  GString *modelyear;
+  GString *reg;
+  GString *year;
+  GString *startkm;
+  GString *endkm;
+  GString *yearkm;
+  GString *sumfill;
+  GString *sumprice;
+  GString *avgconsum;
+  GString *avgco2;
+  GString *avgprice;
+  GString *totalco2;
+  GString *pricekm;
+  GString *priceoil;
+  GString *priceoilkm;
+  GString *priceservice;
+  GString *priceservicekm;
+  GString *pricetires;
+  GString *pricetireskm;
+  GString *totalprice;
+  GString *totalpricekm;
+  float totprice;
+  float totkm;
 
  if (arg1 == GTK_RESPONSE_ACCEPT) {
 
+#if MAEMO_VERSION_MAJOR == 5
+/*    lastyearbuf=hildon_button_get_value(HILDON_BUTTON(pui->lastyearbutton)); */
+   lastyearbuf=hildon_touch_selector_get_current_text(HILDON_TOUCH_SELECTOR(pui->yearcombo));
+#else
    lastyearbuf = gtk_combo_box_get_active_text(GTK_COMBO_BOX(pui->yearcombo));
+#endif
    lastyear=atoi(lastyearbuf);
+#if MAEMO_VERSION_MAJOR == 5
+#else
    g_free(lastyearbuf);
+#endif
 
 /*    if (gtk_combo_box_get_active(GTK_COMBO_BOX(pui->papercombo))==0) { */
    if (reportpaper == A4PAPER) {
      page_width = GTK_PLOT_A4_W;
      page_height = GTK_PLOT_A4_H;
+#if MAEMO_VERSION_MAJOR == 5
+     scale = 800.0/GTK_PLOT_A4_W;
+#else
      scale = 670.0/GTK_PLOT_A4_W;
+#endif
    }
    else {
      page_width = GTK_PLOT_LETTER_W;
      page_height = GTK_PLOT_LETTER_H;
+#if MAEMO_VERSION_MAJOR == 5
+     scale = 800.0/GTK_PLOT_LETTER_W;
+#else
      scale = 670.0/GTK_PLOT_LETTER_W;
+#endif
    }
 
    /* Destroy the report settings dialog */
    gtk_widget_destroy(GTK_WIDGET(dialog));
 
-   if (carchanged)
+   if (carchanged) {
      update_car_changed(pui);
+     update_car_combo(pui->stb_carcombo);
+   }
 
 #if PLAINGTK == 1
    pui->app->reportwindow=gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(pui->app->reportwindow), "Report");
    gtk_widget_set_usize(pui->app->reportwindow,800,600);
    gtk_container_border_width(GTK_CONTAINER(pui->app->reportwindow),0);
+#else
+#if MAEMO_VERSION_MAJOR == 5
+  pui->app->reportwindow = hildon_stackable_window_new();
 #else
    pui->app->reportwindow = HILDON_WINDOW(hildon_window_new());
+#endif
    hildon_program_add_window(pui->app->program, pui->app->reportwindow);
 
   /* Callback for hardware keys */
@@ -670,20 +776,32 @@ void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 #endif
 
 
+#if MAEMO_VERSION_MAJOR == 5
+#else
 #if PLAINGTK == 1
    create_report_toolbar(pui->app->reportwindow, vbox1, pui->reportcanvas);
 #else
    create_report_toolbar(pui->app->reportwindow, pui->reportcanvas);
 #endif
+#endif
 
+#if MAEMO_VERSION_MAJOR == 5
+  pui->reportscrollwin = hildon_pannable_area_new();
+  g_object_set(G_OBJECT(pui->reportscrollwin), "mov-mode", HILDON_MOVEMENT_MODE_BOTH, NULL);
+#else
    pui->reportscrollwin=gtk_scrolled_window_new(NULL, NULL);
    gtk_container_border_width(GTK_CONTAINER(pui->reportscrollwin),0);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(pui->reportscrollwin),
 				  GTK_POLICY_ALWAYS,GTK_POLICY_ALWAYS);
+#endif
    gtk_box_pack_start(GTK_BOX(vbox1),pui->reportscrollwin, TRUE, TRUE,0);
    gtk_widget_show(pui->reportscrollwin);
 
+#if MAEMO_VERSION_MAJOR == 5
+  hildon_pannable_area_add_with_viewport(HILDON_PANNABLE_AREA(pui->reportscrollwin), pui->reportcanvas);
+#else
    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(pui->reportscrollwin), pui->reportcanvas);
+#endif
 
    gtk_widget_show(pui->reportcanvas);
 
@@ -697,13 +815,36 @@ void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 		   HEADING2SIZE, GTK_JUSTIFY_LEFT, gettext(cartexts[0]));
    texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);
 
+   car=g_string_new(NULL);
+   modelyear=g_string_new(NULL);
+   reg=g_string_new(NULL);
+   year=g_string_new(NULL);
+   startkm=g_string_new(NULL);
+   endkm=g_string_new(NULL);
+   yearkm=g_string_new(NULL);
+   sumfill=g_string_new(NULL);
+   sumprice=g_string_new(NULL);
+   avgconsum=g_string_new(NULL);
+   avgco2=g_string_new(NULL);
+   avgprice=g_string_new(NULL);
+   totalco2=g_string_new(NULL);
+   pricekm=g_string_new(NULL);
+   priceoil=g_string_new(NULL);
+   priceoilkm=g_string_new(NULL);
+   priceservice=g_string_new(NULL);
+   priceservicekm=g_string_new(NULL);
+   pricetires=g_string_new(NULL);
+   pricetireskm=g_string_new(NULL);
+   totalprice=g_string_new(NULL);
+   totalpricekm=g_string_new(NULL);
+
    if (SQLITE_OK == sqlite3_bind_int(ppStmtCurCar, 1, currentcar)) {
      if (SQLITE_ROW == sqlite3_step(ppStmtCurCar)) {
-       g_snprintf(car, 30, "%s %s", 
+       g_string_printf(car, "%s %s", 
 		  sqlite3_column_text(ppStmtCurCar, 0),
 		  sqlite3_column_text(ppStmtCurCar, 1));
-       g_snprintf(reg, 30, "%s", sqlite3_column_text(ppStmtCurCar, 2));
-       g_snprintf(modelyear, 30, "%s", sqlite3_column_text(ppStmtCurCar, 3));
+       g_string_printf(reg, "%s", sqlite3_column_text(ppStmtCurCar, 2));
+       g_string_printf(modelyear, "%s", sqlite3_column_text(ppStmtCurCar, 3));
      }
      sqlite3_reset(ppStmtCurCar);
    }
@@ -754,7 +895,6 @@ void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
    for (i=1;i<sizeof(oiltexts)/sizeof(char *);i++) {
      char *str;
      str=report_conv_cost_label(gettext(oiltexts[i]));
-     g_print("%s\n",str);
      report_put_text(pui->reportcanvas,
 		     LEFTMARGIN+INDENT, texty, "Helvetica", NORMALSIZE, GTK_JUSTIFY_LEFT,
 		     str);
@@ -817,59 +957,61 @@ void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
    texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);
    report_put_text(pui->reportcanvas, FIRSTCOLUMN, texty, "Helvetica", NORMALSIZE,
 		   GTK_JUSTIFY_LEFT,
-		   car);
+		   car->str);
    texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
    report_put_text(pui->reportcanvas, FIRSTCOLUMN, texty, "Helvetica", NORMALSIZE,
 		   GTK_JUSTIFY_LEFT,
-		   modelyear);
+		   modelyear->str);
    texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
    report_put_text(pui->reportcanvas, FIRSTCOLUMN, texty, "Helvetica", NORMALSIZE,
 		   GTK_JUSTIFY_LEFT,
-		   reg);
+		   reg->str);
    texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 
    /* Fill the column with data */
-   if (SQLITE_OK == sqlite3_bind_int(ppStmtGetReport, 1, currentcar)) {
+   if (SQLITE_OK == db_init_report_query(currentcar)) {
      textx = FIRSTCOLUMN;
-     while(SQLITE_ROW == sqlite3_step(ppStmtGetReport)) {
-       g_snprintf(year, ENTRYDATEMAX, "%s",
-		  sqlite3_column_text(ppStmtGetReport,0));
-       if (atoi(year) > lastyear) break;
-       if (atoi(year) > lastyear-REPORTYEARS) {
-	 g_snprintf(startkm, ENTRYKMMAX, "%.0f",
-		    SIlength2user(sqlite3_column_double(ppStmtGetReport,1)));
-	 g_snprintf(endkm, ENTRYKMMAX, "%.0f",
-		    SIlength2user(sqlite3_column_double(ppStmtGetReport,2)));
-	 g_snprintf(yearkm, ENTRYKMMAX, "%.0f",
-		    SIlength2user(sqlite3_column_double(ppStmtGetReport,3)));
-	 g_snprintf(sumfill, ENTRYKMMAX, "%.1f",
-		    SIvolume2user(sqlite3_column_double(ppStmtGetReport,4)));
-	 g_snprintf(sumprice, ENTRYKMMAX, "%.1f",
-		    sqlite3_column_double(ppStmtGetReport,5));
-	 g_snprintf(avgconsum, ENTRYKMMAX, "%.1f",
-		    SIconsumption2user(sqlite3_column_double(ppStmtGetReport,6)));
-	 g_snprintf(avgco2, ENTRYKMMAX, "%.0f",
-		    SIemission2user(sqlite3_column_double(ppStmtGetReport,6)*
-				    get_emission_per_litre(db_get_fueltype(currentcar))/100));
-	 g_snprintf(totalco2, ENTRYKMMAX, "%.0f",
-		    SIemission2user(sqlite3_column_double(ppStmtGetReport,4)*
-				    get_emission_per_litre(db_get_fueltype(currentcar))/1000));
-	 g_snprintf(avgprice, ENTRYKMMAX, "%.2f",
-		    SIppl2user(sqlite3_column_double(ppStmtGetReport,7)));
-	 g_snprintf(pricekm, ENTRYKMMAX, "%.2f",
-		    user2SIlength(sqlite3_column_double(ppStmtGetReport,8)));
-	 g_snprintf(priceoil, ENTRYKMMAX, "%.2f",
-		    sqlite3_column_double(ppStmtGetReport,9));
-	 g_snprintf(priceoilkm, ENTRYKMMAX, "%.2f",
-		    user2SIlength(sqlite3_column_double(ppStmtGetReport,10)));
-	 g_snprintf(priceservice, ENTRYKMMAX, "%.2f",
-		    sqlite3_column_double(ppStmtGetReport,11));
-	 g_snprintf(priceservicekm, ENTRYKMMAX, "%.2f",
-		    user2SIlength(sqlite3_column_double(ppStmtGetReport,12)));
-	 g_snprintf(pricetires, ENTRYKMMAX, "%.2f",
-		    sqlite3_column_double(ppStmtGetReport,13));
-	 g_snprintf(pricetireskm, ENTRYKMMAX, "%.2f",
-		    user2SIlength(sqlite3_column_double(ppStmtGetReport,14)));
+     while(SQLITE_ROW == db_step_report_query()) {
+       g_string_printf(year, "%s",
+		       db_report_column_text(0));
+       if (atoi(year->str) == 0 || 
+	   (atoi(year->str) > lastyear-REPORTYEARS) &&
+	   (atoi(year->str) <= lastyear)) {
+	 totkm=SIlength2user(db_report_column_double(3));
+	 g_string_printf(startkm, "%.0f",
+			 SIlength2user(db_report_column_double(1)));
+	 g_string_printf(endkm, "%.0f",
+			 SIlength2user(db_report_column_double(2)));
+	 g_string_printf(yearkm, "%.0f",
+			 totkm);
+	 g_string_printf(sumfill, "%.1f",
+			 SIvolume2user(db_report_column_double(4)));
+	 g_string_printf(sumprice, "%.1f",
+			 db_report_column_double(5));
+	 g_string_printf(avgconsum, "%.1f",
+			 SIconsumption2user(db_report_column_double(6)));
+	 g_string_printf(avgco2, "%.0f",
+			 SIemission2user(db_report_column_double(6)*
+					 get_emission_per_litre(db_get_fueltype(currentcar))/100));
+	 g_string_printf(totalco2, "%.0f",
+			 SImass2user(grams2kg(db_report_column_double(4)*
+					      get_emission_per_litre(db_get_fueltype(currentcar)))));
+	 g_string_printf(avgprice, "%.2f",
+			 SIppl2user(db_report_column_double(7)));
+	 g_string_printf(pricekm, "%.2f",
+			 user2SIlength(db_report_column_double(8)));
+	 g_string_printf(priceoil, "%.2f",
+			 db_report_column_double(9));
+	 g_string_printf(priceoilkm, "%.2f",
+			 user2SIlength(db_report_column_double(10)));
+	 g_string_printf(priceservice, "%.2f",
+			 db_report_column_double(11));
+	 g_string_printf(priceservicekm, "%.2f",
+			 user2SIlength(db_report_column_double(12)));
+	 g_string_printf(pricetires, "%.2f",
+			 db_report_column_double(13));
+	 g_string_printf(pricetireskm, "%.2f",
+			 user2SIlength(db_report_column_double(14)));
 
 	 /* Year header*/
 	 texty=report_init_line(HEADERLINE);
@@ -878,18 +1020,18 @@ void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 	 texty=report_line_down(texty, NORMALSIZE, ((sizeof(cartexts)/sizeof(char *))-1)*1.2, page_height);
 	 texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, year);
+			 GTK_JUSTIFY_LEFT, year->str);
 	 texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);
 
 	 /* Km */
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, startkm);
+			 GTK_JUSTIFY_LEFT, startkm->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, endkm);
+			 GTK_JUSTIFY_LEFT, endkm->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, yearkm);
+			 GTK_JUSTIFY_LEFT, yearkm->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 
 	 texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
@@ -897,25 +1039,25 @@ void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 
 	 /* Fuel */
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, sumfill);
+			 GTK_JUSTIFY_LEFT, sumfill->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, sumprice);
+			 GTK_JUSTIFY_LEFT, sumprice->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, avgconsum);
+			 GTK_JUSTIFY_LEFT, avgconsum->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, avgco2);
+			 GTK_JUSTIFY_LEFT, avgco2->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, totalco2);
+			 GTK_JUSTIFY_LEFT, totalco2->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, avgprice);
+			 GTK_JUSTIFY_LEFT, avgprice->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, pricekm);
+			 GTK_JUSTIFY_LEFT, pricekm->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 
 	 texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
@@ -923,10 +1065,10 @@ void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 
 	 /* Oil */
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, priceoil);
+			 GTK_JUSTIFY_LEFT, priceoil->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, priceoilkm);
+			 GTK_JUSTIFY_LEFT, priceoilkm->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 
 	 texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
@@ -934,10 +1076,10 @@ void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 
 	 /* Service */
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, priceservice);
+			 GTK_JUSTIFY_LEFT, priceservice->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, priceservicekm);
+			 GTK_JUSTIFY_LEFT, priceservicekm->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 
 	 texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
@@ -945,34 +1087,39 @@ void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 
 	 /* Tires */
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, pricetires);
+			 GTK_JUSTIFY_LEFT, pricetires->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, pricetireskm);
+			 GTK_JUSTIFY_LEFT, pricetireskm->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 
 	 texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
 	 texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);
 
 	 /* Overall expenses */
-	 g_snprintf(totalprice, ENTRYDATEMAX, "%.2f", 
-		    atof(sumprice)+atof(priceoil)+atof(priceservice)
-		    +atof(pricetires));
+	 totprice=atof(sumprice->str)
+	   +atof(priceoil->str)+atof(priceservice->str)
+	   +atof(pricetires->str);
+	 g_string_printf(totalprice, "%.2f", totprice);
+	 g_string_printf(totalpricekm, "%.2f", totprice/SIlength2user(totkm));
 
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, sumprice);
+			 GTK_JUSTIFY_LEFT, sumprice->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, priceoil);
+			 GTK_JUSTIFY_LEFT, priceoil->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, priceservice);
+			 GTK_JUSTIFY_LEFT, priceservice->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, pricetires);
+			 GTK_JUSTIFY_LEFT, pricetires->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
-			 GTK_JUSTIFY_LEFT, totalprice);
+			 GTK_JUSTIFY_LEFT, totalprice->str);
+	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
+	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
+			 GTK_JUSTIFY_LEFT, totalpricekm->str);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 
 
@@ -980,11 +1127,34 @@ void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 	 textx += COLUMNSEP;
        }
      }
-     sqlite3_reset(ppStmtGetReport);
+     db_reset_report_query();
    }
    else
      g_print("Something wrong in obtaining the report data\n");
+   db_close_report_query();
 
+   g_string_free(car, TRUE);
+   g_string_free(modelyear, TRUE);
+   g_string_free(reg, TRUE);
+   g_string_free(year, TRUE);
+   g_string_free(startkm, TRUE);
+   g_string_free(endkm, TRUE);
+   g_string_free(yearkm, TRUE);
+   g_string_free(sumfill, TRUE);
+   g_string_free(sumprice, TRUE);
+   g_string_free(avgconsum, TRUE);
+   g_string_free(avgco2, TRUE);
+   g_string_free(avgprice, TRUE);
+   g_string_free(totalco2, TRUE);
+   g_string_free(pricekm, TRUE);
+   g_string_free(priceoil, TRUE);
+   g_string_free(priceoilkm, TRUE);
+   g_string_free(priceservice, TRUE);
+   g_string_free(priceservicekm, TRUE);
+   g_string_free(pricetires, TRUE);
+   g_string_free(pricetireskm, TRUE);
+   g_string_free(totalprice, TRUE);
+   g_string_free(totalpricekm, TRUE);
 
    gtk_widget_show_all(GTK_WIDGET(pui->app->reportwindow));
    if (pui->mainfullscreen)
@@ -1036,30 +1206,59 @@ void callback_report( GtkAction * action, AppUIData *pui )
     table = gtk_table_new(2, 3, FALSE);
     gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 5);
 
+#if MAEMO_VERSION_MAJOR == 5
+  pui->carbutton = hildon_picker_button_new (HILDON_SIZE_AUTO, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
+  hildon_button_set_title (HILDON_BUTTON (pui->carbutton), _("Car"));
+  gtk_table_attach(GTK_TABLE(table), pui->carbutton, 0, 3, 0, 1, 
+		   GTK_EXPAND|GTK_FILL,
+		   0, 0, 5);
+  gtk_widget_show(pui->carbutton);
+#else
     label=gtk_label_new(_("Car:"));
     gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, 
 		     GTK_EXPAND|GTK_FILL,
 		     0, 0, 5);
     gtk_widget_show(label);
+#endif
 
-    pui->carcombo = create_car_combo();
+#if MAEMO_VERSION_MAJOR == 5
+    pui->carcombo = create_car_combo(pui, pui->carbutton);
+#else
+    pui->carcombo = create_car_combo(pui);
     gtk_table_attach(GTK_TABLE(table), pui->carcombo, 1, 2, 0, 1, 
 		     GTK_EXPAND|GTK_FILL,
 		     0, 0, 5);
     gtk_widget_show(pui->carcombo);
+#endif
 
+#if MAEMO_VERSION_MAJOR == 5
+    pui->lastyearbutton = hildon_picker_button_new (HILDON_SIZE_AUTO, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
+    hildon_button_set_title (HILDON_BUTTON (pui->lastyearbutton), _("Last year"));
+    gtk_table_attach(GTK_TABLE(table), pui->lastyearbutton, 0, 3, 1, 2, 
+		     GTK_EXPAND|GTK_FILL,
+		     0, 0, 5);
+    gtk_widget_show(pui->lastyearbutton);
+#else
     label=gtk_label_new(_("Last year:"));
     gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 
 		     GTK_EXPAND|GTK_FILL,
 		     0, 0, 5);
     gtk_widget_show(label);
+#endif
 
-    pui->yearcombo = create_year_combo();
+    pui->yearcombo = create_year_combo(pui);
+#if MAEMO_VERSION_MAJOR == 5
+#else
     gtk_table_attach(GTK_TABLE(table), pui->yearcombo, 1, 2, 1, 2,
 		     GTK_EXPAND|GTK_FILL,
 		     0, 0, 5);
     gtk_widget_show(pui->yearcombo);
+#endif
 
+#if MAEMO_VERSION_MAJOR == 5
+    g_signal_connect(G_OBJECT(pui->carbutton), "value-changed",
+		     G_CALLBACK(update_yearcombo_cb), pui->yearcombo);
+#else
     button = gtk_button_new_with_label(_("Update"));
     gtk_table_attach(GTK_TABLE(table), button, 2, 3, 1, 2,
 		     GTK_EXPAND|GTK_FILL,
@@ -1067,12 +1266,13 @@ void callback_report( GtkAction * action, AppUIData *pui )
     gtk_widget_show(button);
     g_signal_connect (GTK_OBJECT (button), "clicked",
 			G_CALLBACK(update_yearcombo_cb), pui->yearcombo);
+#endif
 
     gtk_widget_show(table);
 
   }
 
-#if PLAINGTK == 0
+#if LIBHELP == 1
   help_dialog_help_enable(GTK_DIALOG(dialog),
 				 HELP_ID_REPORT,
 				 pui->app->osso);
